package provide tticket 0.1

package require tticketProduct
package require tticketUser
package require tticketTicket

namespace eval ::tticket {
    # procs defined here
    namespace export login logout
    namespace export listPlatforms
    namespace export listPriorities getPriorityDescription
    namespace export listCategories getCategoryDescription

    # variable holding connection info
    variable connections
}

#
# tticket has its own user and db handling
# on login the username and resulting database handle
# is stored in the 'connections' array. The returned username
# is further on used for all procs as first parameter
#

# Create the db connection
proc ::tticket::login {user password {host localhost}} {
    variable connections

    set conninfo [list user=$user password=$password \
		      host=$host dbname=tticket]
    set db [pg_connect -conninfo $conninfo]
    
    # create a new userID
    set userID "tticket[clock clicks]"
    set connections($userID,h) $db
    set connections($userID,user) $user
    return $userID
}

# return a db handle for a user name
proc ::tticket::resolveDB {userID} {
    variable connections

    if {[catch {
	set db $connections($userID,h)
    }]} {
	error "invalid userID '$userID', please login first"
    }
    return $db
}

# Close the db connection
proc ::tticket::logout {userID} {
    catch {
	set db [resolveDB $userID]
	pg_disconnect $db
    }
}

#
# Platforms
#

# list available platforms
proc ::tticket::listPlatforms {userID} {
    set db [resolveDB $userID]

    set sql "select name from platforms"
    set h [pg_exec $db $sql]

    # error handling
    if {[string length [set msg [pg_result $h -error]]]} {
	pg_result $h -clear
	error "error listing products: $msg"
    }

    for {set i 0} {$i < [pg_result $h -numTuples]} {incr i} {
	lappend retval [join [join [pg_result $h -getTuple $i]]]
    }
    pg_result $h -clear

    return $retval

}

#
# Priorities
#

# list available priorities
proc ::tticket::listPriorities {userID} {

    set db [resolveDB $userID]

    set sql "select name from priorities"
    set h [pg_exec $db $sql]

    # error handling
    if {[string length [set msg [pg_result $h -error]]]} {
	pg_result $h -clear
	error "error listing products: $msg"
    }

    for {set i 0} {$i < [pg_result $h -numTuples]} {incr i} {
	lappend retval [pg_result $h -getTuple $i]
    }
    pg_result $h -clear

    return $retval

}

# get priority description
proc ::tticket::getPriorityDescription {userID name} {
    set db [resolveDB $userID]
    set sql "select name, description from priorities where name='$name'"
    set h [pg_exec $db $sql]

    # error handling
    if {[string length [set msg [pg_result $h -error]]]} {
	pg_result $h -clear
	error "error getting description: $msg"
    }
    set description [lindex [pg_result $h -getTuple 0] 1]
    pg_result $h -clear 

    return $description
}

#
# Categories
#

# list available categories
proc ::tticket::listCategories {userID} {
    set db [resolveDB $userID]

    set sql "select name from categories"
    set h [pg_exec $db $sql]

    # error handling
    if {[string length [set msg [pg_result $h -error]]]} {
	pg_result $h -clear
	error "error listing products: $msg"
    }

    for {set i 0} {$i < [pg_result $h -numTuples]} {incr i} {
	lappend retval [pg_result $h -getTuple $i]
    }
    pg_result $h -clear

    return $retval

}

# get category description
proc ::tticket::getCategoryDescription {userID name} {
    set db [resolveDB $userID]

    set sql "select name, description from categories where name='$name'"
    set h [pg_exec $db $sql]

    # error handling
    if {[string length [set msg [pg_result $h -error]]]} {
	pg_result $h -clear
	error "error getting description: $msg"
    }
    set description [lindex [pg_result $h -getTuple 0] 1]
    pg_result $h -clear 

    return $description
}

